'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFileMigrateDc = exports.getMigrateDc = exports.isFileMigrateError = exports.isMigrateError = exports.onRpcError = undefined;

var _bin = require('../bin');

var _error = require('../error');

var _mtprotoLogger = require('mtproto-logger');

var _mtprotoLogger2 = _interopRequireDefault(_mtprotoLogger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var log = _mtprotoLogger2.default`event, rpc`;

var onRpcError = exports.onRpcError = rpcError => {
  var errorMessage = rpcError.error_message || '';
  var matches = errorMessage.match(/^([A-Z_0-9]+\b)(: (.+))?/) || [];
  var errorCode = (0, _bin.uintToInt)(rpcError.error_code || 0);
  var code = !errorCode || errorCode <= 0 ? 500 : errorCode;
  var type = matches[1] || 'UNKNOWN';
  var description = matches[3] || `CODE#${code} ${errorMessage}`;
  return new _error.RpcError(code, type, description, rpcError);
};

var migrateRegexp = /^(PHONE_MIGRATE_|NETWORK_MIGRATE_|USER_MIGRATE_)(\d+)/;
var fileMigrateRegexp = /^(FILE_MIGRATE_)(\d+)/;

var isMigrateError = exports.isMigrateError = err => migrateRegexp.test(err.type);
var isFileMigrateError = exports.isFileMigrateError = err => fileMigrateRegexp.test(err.type);

var getMigrateDc = exports.getMigrateDc = (err, regExp = migrateRegexp) => {
  var matched = err.type.match(regExp);
  if (!matched || matched.length < 2) {
    log('warning')('no matched error type', err.type);
    return null;
  }
  var [,, newDcID] = matched;
  if (!isFinite(newDcID)) {
    log('warning', 'migrated error')('invalid dc', newDcID);
    return null;
  }
  var newDc = parseInt(newDcID, 10);
  return newDc;
};

var getFileMigrateDc = exports.getFileMigrateDc = err => getMigrateDc(err, fileMigrateRegexp);
//# sourceMappingURL=rpc.js.map